﻿using System.DirectoryServices;
using System.Globalization;
using System.IO;
using System.Text;
using System.Text.Json;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace RobotTestGraphical
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private readonly RobotProgram robotProgram;
        private UniversalRobot robot;
        private const string settingsJsonFilename = "settings.json";
        private bool settingsLoaded = false;
        private void Log(string line)
        {
            // If we don't use the following construct, the robot operation task cannot log messages

            // Check if the current thread is the UI thread
            if (Dispatcher.CheckAccess())
            {
                // Directly update the UI
                Logs.AppendText(line + Environment.NewLine);
            }
            else
            {
                // Use the Dispatcher to invoke the update on the UI thread
                Dispatcher.Invoke(() =>
                {
                    Logs.AppendText(line + Environment.NewLine);
                });
            }
        }
        private void OnNewLogMessageCreated(object? sender, string e)
        {
            Log(e);
        }
        public MainWindow()
        {
            InitializeComponent();
            Thread.CurrentThread.CurrentCulture = new CultureInfo("en-US");
            LoadSettingsFromJson();
            robot = new UniversalRobot(Licensee.Text, LicenseKey.Text, IPAddress.Text);
            robotProgram = new RobotProgram(robot);
            robotProgram.NewLogMessageCreated += OnNewLogMessageCreated;
            Log($"Started @ {DateTime.Now}\n");
        }
        private void ConnectClicked(object sender, RoutedEventArgs e)
        {
            try
            {
                robot.ActivateRobot();
            }
            catch (Exception ex)
            {
                Log(ex.Message);
                return;
            }
            Log("✅ Robot connection established");
            Disconnect.IsEnabled = true;
            Start.IsEnabled = true;
            Stop.IsEnabled = true;
        }
        private void DisconnectClicked(object sender, RoutedEventArgs e)
        {
            robot.Disconnect();
            Disconnect.IsEnabled = false;
            Start.IsEnabled = false;
            Stop.IsEnabled = false;
        }
        private void SettingsChanged(object sender, System.Windows.Controls.TextChangedEventArgs e)
        {
            if (settingsLoaded)  // Otherwise UI elements deliver empty data.
            {
                SaveSettingsToJson();
                robot = new UniversalRobot(Licensee.Text, LicenseKey.Text, IPAddress.Text);
            }
        }
        private void LoadSettingsFromJson()
        {
            settingsLoaded = true;
            if (!File.Exists(settingsJsonFilename))
                return; // If file doesn't exist, skip loading

            // Read the JSON file and deserialize into a dictionary
            var json = File.ReadAllText(settingsJsonFilename);
            var data = JsonSerializer.Deserialize<Dictionary<string, string>>(json);

            if (data == null) return;

            // Iterate over the fields and set their values
            foreach (var keyValuePair in data)
            {
                // Json file could contain TextBox names which do not actually exist. Check this.
                if (FindName(keyValuePair.Key) is TextBox textBox)
                    textBox.Text = keyValuePair.Value; // Set the field value
            }
        }
        private readonly JsonSerializerOptions jsonSerializerOptions = new() { WriteIndented = true };
        private void SaveSettingsToJson()
        {
            var tabItem = this.FindName("SettingsGrid") as Grid;
            var settingsDict = new Dictionary<string, string>();

            foreach (var textBox in LogicalTreeHelper.GetChildren(tabItem).OfType<TextBox>())
            {
                settingsDict[textBox.Name] = textBox.Text;
            }
            // Convert settings into JSON and write to a file
            var json = JsonSerializer.Serialize(settingsDict, jsonSerializerOptions);
            File.WriteAllText(settingsJsonFilename, json);
        }
        private void StartClicked(object sender, RoutedEventArgs e)
        {
            Task.Run(() => robotProgram.Run());
        }
        private void StopClicked(object sender, RoutedEventArgs e)
        {
            robot.Stop();
        }
    }
    
}
